/* name:    timestamp.cxx
   author:  Leo Liberti
   purpose: handle timestamps in Unix (implementation)
   source:  GNU C++ 4.1.2
   history: 080218 work started
*/

#include <iostream>
#include <ctime>
#include "timestamp.h"

// implementation of TimeStampException methods
TimeStampException::TimeStampException() { }
TimeStampException::~TimeStampException() { }

// implementation of TimeStamp methods
TimeStamp::TimeStamp() : timestamp(0) { }
TimeStamp::~TimeStamp() { }

long TimeStamp::get(void) const {
  return timestamp;
}

void TimeStamp::set(long thetimestamp) {
  timestamp = thetimestamp;
}

void TimeStamp::update(void) throw (TimeStampException) {
  struct timeval tv;
  struct timezone tz;
  using namespace std;
  try {
    int retVal = gettimeofday(&tv, &tz);
    if (retVal == -1) {
      cerr << "TimeStamp::update(): couldn't get system time" << endl;
      throw TimeStampException();
    }
  } catch (...) {
    cerr << "TimeStamp::update(): couldn't get system time" << endl;
    throw TimeStampException();
  }
  timestamp = tv.tv_sec;
}

std::ostream& operator<<(std::ostream& s, TimeStamp& t) 
  throw (TimeStampException) {
  using namespace std;
  time_t theTime = (time_t) t.get();
  char* buffer;
  try {
    buffer = ctime(&theTime);
  } catch (...) {
    cerr << "TimeStamp::update(): couldn't print system time" << endl;
    throw TimeStampException();
  }
  buffer[strlen(buffer) - 1] = '\0';
  s << buffer;
  return s;
}
